import { defineComponent, ref, computed, resolveComponent, openBlock, createBlock, unref, withCtx, createElementBlock, Fragment, renderList, toDisplayString, resolveDynamicComponent, mergeProps, createElementVNode, createVNode, createTextVNode, createCommentVNode, normalizeClass } from "vue";
import { I as InlineSvg, O as svgPath, Q as getCommands } from "./kit-ddc495fb.js";
import { c as createKeysLocalStorageLRUCache } from "./lru-cache-02c096c4.js";
import "element-plus";
const _hoisted_1 = {
  key: 0,
  class: "fs12px lh-16px color-gray-400 pt-8px px-16px pb-4px"
};
const __default__ = {
  name: "tiptap-menus-insert"
};
const _sfc_main = /* @__PURE__ */ defineComponent({
  ...__default__,
  props: {
    expand: null,
    editor: null
  },
  setup(__props) {
    const props = __props;
    const storageKey = `DOC_EDITOR_INSERT`;
    const insertLocalStorageLRUCache = createKeysLocalStorageLRUCache(storageKey, 3);
    const runCommand = (command) => {
      insertLocalStorageLRUCache.put(command.label);
      recentUsed.value = insertLocalStorageLRUCache.get();
      command.action(props.editor);
    };
    const commands = getCommands(runCommand);
    props.expand.forEach((item) => {
      commands.splice(
        [null, void 0].some((type) => item.index === type) ? commands.length : item.index,
        0,
        item
      );
    });
    const recentUsed = ref(
      (() => {
        insertLocalStorageLRUCache.get().forEach((item) => {
          if (!commands.some((command) => command.label === item)) {
            insertLocalStorageLRUCache.remove(item);
          }
        });
        return insertLocalStorageLRUCache.get();
      })()
    );
    const isShow = ref(false);
    const isTitleActive = computed(() => {
      var _a;
      return (_a = props.editor) == null ? void 0 : _a.isActive.apply(props.editor, ["title"]);
    });
    const renderedList = computed(() => {
      return recentUsed.value.length ? [
        {
          title: "最近使用"
        },
        ...recentUsed.value.map((label) => {
          return commands.find((data) => {
            return data.label === label;
          });
        }).filter((command) => {
          return (command == null ? void 0 : command.label) && (command.label === "表格" || command.label === "布局") ? Boolean(command) : true;
        }),
        ...commands
      ] : commands;
    });
    return (_ctx, _cache) => {
      const _component_el_tooltip = resolveComponent("el-tooltip");
      const _component_el_dropdown_item = resolveComponent("el-dropdown-item");
      const _component_el_dropdown_menu = resolveComponent("el-dropdown-menu");
      const _component_el_dropdown = resolveComponent("el-dropdown");
      return openBlock(), createBlock(_component_el_dropdown, {
        trigger: "click",
        "popper-class": "none-duration",
        disabled: unref(isTitleActive),
        onVisibleChange: _cache[0] || (_cache[0] = (val) => isShow.value = val)
      }, {
        dropdown: withCtx(() => [
          isShow.value ? (openBlock(), createBlock(_component_el_dropdown_menu, { key: 0 }, {
            default: withCtx(() => [
              (openBlock(true), createElementBlock(Fragment, null, renderList(unref(renderedList), (item, index) => {
                return openBlock(), createElementBlock(Fragment, { key: index }, [
                  item.title ? (openBlock(), createElementBlock("div", _hoisted_1, toDisplayString(item.title), 1)) : item.custom ? (openBlock(), createBlock(resolveDynamicComponent(item.custom), mergeProps({
                    key: 1,
                    editor: __props.editor,
                    "on-select": item.onSelect
                  }, item.props), {
                    default: withCtx(() => [
                      createElementVNode("div", null, [
                        createVNode(_component_el_dropdown_item, null, {
                          default: withCtx(() => [
                            createVNode(unref(InlineSvg), {
                              class: "w-16px h-16px mr-9px mt-2px",
                              src: item.icon
                            }, null, 8, ["src"]),
                            createTextVNode(" " + toDisplayString(item.label), 1)
                          ]),
                          _: 2
                        }, 1024)
                      ])
                    ]),
                    _: 2
                  }, 1040, ["editor", "on-select"])) : (openBlock(), createBlock(_component_el_dropdown_item, {
                    key: 2,
                    onClick: ($event) => runCommand(item)
                  }, {
                    default: withCtx(() => [
                      createVNode(unref(InlineSvg), {
                        class: "w-16px h-16px mr-9px mt-2px",
                        src: item.icon
                      }, null, 8, ["src"]),
                      createTextVNode(" " + toDisplayString(item.label), 1)
                    ]),
                    _: 2
                  }, 1032, ["onClick"]))
                ], 64);
              }), 128))
            ]),
            _: 1
          })) : createCommentVNode("", true)
        ]),
        default: withCtx(() => [
          createElementVNode("div", null, [
            createVNode(_component_el_tooltip, {
              content: "插入",
              effect: "dark",
              placement: "top",
              enterable: false,
              "hide-after": 0
            }, {
              default: withCtx(() => [
                createElementVNode("button", {
                  class: normalizeClass(["tiptap-button", { "bg-hover-secondary": !unref(isTitleActive), "text-gray-500": unref(isTitleActive) }])
                }, [
                  createVNode(unref(InlineSvg), {
                    class: "w-16px h-16px",
                    src: unref(svgPath)
                  }, null, 8, ["src"])
                ], 2)
              ]),
              _: 1
            })
          ])
        ]),
        _: 1
      }, 8, ["disabled"]);
    };
  }
});
export {
  _sfc_main as default
};
