var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
import { nextTick, defineComponent, ref, onMounted, onBeforeUnmount, openBlock, createElementBlock, renderSlot } from "vue";
import { a as Plugin, P as PluginKey, a4 as isTextSelection, q as tippy, a5 as isNodeSelection, a6 as posToDOMRect } from "./kit-ddc495fb.js";
const ACTIVE_BUBBLE_MENUS = [];
class BubbleMenuView {
  constructor({
    editor,
    element,
    view,
    tippyOptions = {},
    shouldShow,
    getRenderContainer,
    defaultAnimation = false
  }) {
    __publicField(this, "editor");
    __publicField(this, "element");
    __publicField(this, "view");
    __publicField(this, "preventHide", false);
    __publicField(this, "tippy");
    __publicField(this, "tippyOptions");
    __publicField(this, "getRenderContainer");
    __publicField(this, "defaultAnimation");
    __publicField(this, "shouldShow", ({ view, state, from, to }) => {
      const { doc, selection } = state;
      const { empty } = selection;
      const isEmptyTextBlock = !doc.textBetween(from, to).length && isTextSelection(state.selection);
      const isChildOfMenu = this.element.contains(document.activeElement);
      const hasEditorFocus = (view == null ? void 0 : view.hasFocus()) || isChildOfMenu;
      if (!hasEditorFocus || empty || isEmptyTextBlock || !this.editor.isEditable) {
        return false;
      }
      return true;
    });
    __publicField(this, "mousedownHandler", () => {
      this.preventHide = true;
    });
    __publicField(this, "dragstartHandler", () => {
      this.hide();
    });
    __publicField(this, "focusHandler", () => {
      setTimeout(() => this.update(this.editor.view));
    });
    __publicField(this, "blurHandler", ({ event }) => {
      var _a;
      if (this.preventHide) {
        this.preventHide = false;
        return;
      }
      if ((event == null ? void 0 : event.relatedTarget) && ((_a = this.element.parentNode) == null ? void 0 : _a.contains(event.relatedTarget))) {
        return;
      }
      this.hide();
    });
    __publicField(this, "addActiveBubbleMenu", () => {
      const idx = ACTIVE_BUBBLE_MENUS.findIndex((instance) => {
        var _a;
        return (instance == null ? void 0 : instance.id) === ((_a = this.tippy) == null ? void 0 : _a.id);
      });
      if (idx < 0) {
        ACTIVE_BUBBLE_MENUS.push(this.tippy);
      }
    });
    __publicField(this, "removeActiveBubbleMenu", () => {
      const idx = ACTIVE_BUBBLE_MENUS.findIndex((instance) => {
        var _a;
        return (instance == null ? void 0 : instance.id) === ((_a = this.tippy) == null ? void 0 : _a.id);
      });
      if (idx > -1) {
        ACTIVE_BUBBLE_MENUS.splice(idx, 1);
      }
    });
    this.editor = editor;
    this.element = element;
    this.view = view;
    this.getRenderContainer = getRenderContainer;
    this.defaultAnimation = defaultAnimation;
    if (shouldShow) {
      this.shouldShow = shouldShow;
    }
    this.element.addEventListener("mousedown", this.mousedownHandler, {
      capture: true
    });
    this.view.dom.addEventListener("dragstart", this.dragstartHandler);
    this.tippyOptions = tippyOptions || {};
    this.element.remove();
    this.element.style.visibility = "visible";
  }
  createTooltip() {
    const { element: editorElement } = this.editor.options;
    const editorIsAttached = !!editorElement.parentElement;
    if (this.tippy || !editorIsAttached) {
      return;
    }
    this.tippy = tippy(editorElement, {
      getReferenceClientRect: null,
      content: this.element,
      interactive: true,
      trigger: "manual",
      placement: "top",
      hideOnClick: "toggle",
      ...Object.assign(
        {
          zIndex: 999,
          duration: 200,
          ...this.defaultAnimation ? { animation: "shift-toward-subtle", moveTransition: "transform 0.2s ease-in-out" } : {}
        },
        this.tippyOptions
      )
    });
    if (this.tippy.popper.firstChild) {
      this.tippy.popper.firstChild.addEventListener("blur", (event) => {
        this.blurHandler({ event });
      });
    }
  }
  update(view, oldState) {
    nextTick(() => {
      var _a, _b, _c, _d, _e, _f;
      const { state, composing } = view;
      const { doc, selection } = state;
      const isSame = oldState && oldState.doc.eq(doc) && oldState.selection.eq(selection);
      if (composing || isSame) {
        return;
      }
      this.createTooltip();
      const { ranges } = selection;
      const cursorAt = selection.$anchor.pos;
      const from = Math.min(...ranges.map((range) => range.$from.pos));
      const to = Math.max(...ranges.map((range) => range.$to.pos));
      const placement = isNodeSelection(selection) ? "top" : Math.abs(cursorAt - to) <= Math.abs(cursorAt - from) ? "bottom-start" : "top-start";
      const domAtPos = view.domAtPos(from).node;
      const nodeDOM = view.nodeDOM(from);
      const node = nodeDOM || domAtPos;
      const shouldShow = this.editor.isEditable && ((_a = this.shouldShow) == null ? void 0 : _a.call(this, {
        editor: this.editor,
        view,
        node,
        state,
        oldState,
        from,
        to
      }));
      if (!shouldShow) {
        this.hide();
        return;
      }
      const otherBubbleMenus = ACTIVE_BUBBLE_MENUS.filter(
        (instance) => {
          var _a2;
          return instance.id !== ((_a2 = this.tippy) == null ? void 0 : _a2.id) && instance.popperInstance && instance.popperInstance.state;
        }
      );
      const offsetX = ((_c = (_b = this.tippyOptions) == null ? void 0 : _b.offset) == null ? void 0 : _c[0]) ?? 0;
      const offsetY = otherBubbleMenus.length ? otherBubbleMenus.reduce((prev, instance, currentIndex, array) => {
        var _a2, _b2, _c2, _d2, _e2, _f2, _g, _h, _i, _j, _k, _l, _m;
        const prevY = array[currentIndex - 1] ? ((_e2 = (_d2 = (_c2 = (_b2 = (_a2 = array[currentIndex - 1]) == null ? void 0 : _a2.popperInstance) == null ? void 0 : _b2.state) == null ? void 0 : _c2.modifiersData) == null ? void 0 : _d2.popperOffsets) == null ? void 0 : _e2.y) ?? 0 : 0;
        const currentY = ((_i = (_h = (_g = (_f2 = instance == null ? void 0 : instance.popperInstance) == null ? void 0 : _f2.state) == null ? void 0 : _g.modifiersData) == null ? void 0 : _h.popperOffsets) == null ? void 0 : _i.y) ?? 0;
        const currentHeight = ((_m = (_l = (_k = (_j = instance == null ? void 0 : instance.popperInstance) == null ? void 0 : _j.state) == null ? void 0 : _k.rects) == null ? void 0 : _l.popper) == null ? void 0 : _m.height) ?? 42;
        if (Math.abs(prevY - currentY) <= currentHeight - 5) {
          prev += currentHeight;
        }
        return prev;
      }, 0) : (
        // @ts-ignore
        ((_e = (_d = this.tippyOptions) == null ? void 0 : _d.offset) == null ? void 0 : _e[1]) ?? 0
      );
      (_f = this.tippy) == null ? void 0 : _f.setProps({
        offset: [offsetX, offsetY + 10],
        placement,
        getReferenceClientRect: () => {
          let toMountNode = null;
          if (isNodeSelection(state.selection)) {
            if (this.getRenderContainer && node) {
              toMountNode = this.getRenderContainer(node);
            }
          }
          if (this.getRenderContainer && node) {
            toMountNode = this.getRenderContainer(node);
          }
          if (toMountNode && toMountNode.getBoundingClientRect) {
            return toMountNode.getBoundingClientRect();
          }
          if (node && node.getBoundingClientRect) {
            return node.getBoundingClientRect();
          }
          return posToDOMRect(view, from, to);
        }
      });
      this.show();
    });
  }
  show() {
    var _a;
    this.addActiveBubbleMenu();
    (_a = this.tippy) == null ? void 0 : _a.show();
  }
  hide() {
    var _a;
    this.removeActiveBubbleMenu();
    (_a = this.tippy) == null ? void 0 : _a.hide();
  }
  destroy() {
    var _a;
    this.removeActiveBubbleMenu();
    (_a = this.tippy) == null ? void 0 : _a.destroy();
    this.element.removeEventListener("mousedown", this.mousedownHandler, {
      capture: true
    });
    this.view.dom.removeEventListener("dragstart", this.dragstartHandler);
    this.editor.off("focus", this.focusHandler);
    this.editor.off("blur", this.blurHandler);
  }
}
const BubbleMenuPlugin = (options) => {
  return new Plugin({
    key: typeof options.pluginKey === "string" ? new PluginKey(options.pluginKey) : options.pluginKey,
    view: (view) => new BubbleMenuView({ view, ...options })
  });
};
const __default__ = defineComponent({
  name: "tiptap-bubble-menu"
});
const _sfc_main = /* @__PURE__ */ defineComponent({
  ...__default__,
  props: {
    editor: null,
    pluginKey: { default: "bubbleMenu" },
    tippyOptions: { default: () => {
      return {};
    } },
    defaultAnimation: null,
    shouldShow: { default: null },
    getRenderContainer: null
  },
  setup(__props) {
    const props = __props;
    const element = ref();
    onMounted(() => {
      const plugin = BubbleMenuPlugin({
        pluginKey: props.pluginKey,
        editor: props.editor,
        element: element.value,
        tippyOptions: props.tippyOptions,
        defaultAnimation: props.defaultAnimation,
        shouldShow: props.shouldShow,
        getRenderContainer: props.getRenderContainer
      });
      props.editor.registerPlugin(plugin);
    });
    onBeforeUnmount(() => {
      props.editor.unregisterPlugin(props.pluginKey);
    });
    return (_ctx, _cache) => {
      return openBlock(), createElementBlock("div", {
        class: "border-primary",
        ref_key: "element",
        ref: element,
        style: { "visibility": "hidden" }
      }, [
        renderSlot(_ctx.$slots, "default")
      ], 512);
    };
  }
});
export {
  _sfc_main as _
};
